function [testFiles, transientFiles, fsSpeech, avgLengthFrames] = createTestFiles(varargin)
%%% Makes test files (@16kHz) of concatenated speech files and white noise pauses.
%%%
%%% Returns struct with testfiles containing following data fields:
%%%         _concatenated_:         sound data from concatenation.
%%%         _starts_:               starting indices for silent/non-speech parts.
%%%         _stops_:                stoping indices for silent/non-speech parts.
%%%         _pauses_:               length of randomly selected pauses.
%%%         _segments_:             length of the individual segments constituting the
%%%                                 concatenated signal (in samples @ fs).
%%%         _structure_:            structure of pauses (0: speech; 1: pause).
%%%         _shifts_:               offset for the starting index of the segments.
%%%         _tranStart_:            starting indices for transient parts.
%%%         _tranStop_:             stoping indices for transient parts.
%%%         _randomTrans_:          indicates the order of the transient sounds used.
%%%         _transientNames_:       indicates the names of the transient sounds used.
%%%         _fs_:                   sampling frequency for _concatenated_.
%%%         _transientScaling_:     indicates the scaling of the transient sounds used.
%%%
%%% The _transientFiles_ output is a list of the transient names. This
%%% corresponds to the indices returned in _testFiles.randomTrans_.
%%%
%%% Added 2014.04.04: Random rearrangment of the concatenation structure
%%% shown below:
%%% |---- SPEECH1 ---||--- SPEECH2 ---||--- PAUSE1 ---||--- SPEECH3 ---||--- PAUSE2 ---||--- SPEECH4 ---||--- PAUSE3 ---||--- SPEECH5 ---|

    plotFiles = 0; %%% set to 1 for plot of resulting files with annotation markings

    if nargin < 1
        noFilesTot = 10; %%% number of concated test files desired
    else
        noFilesTot = varargin{1};
        if nargin > 1
            stpsz = varargin{2};
        else
            stpsz = 80;
        end
    end
    noSegmentsInEach = 5; %%% number of speech segments in each test file
    pauseLength = [3, 30]; %%% [min max] length of pause between segments.
    filesNeeded = noFilesTot*noSegmentsInEach;
    fullfile = {}; %%% pre-declared to override the built-in function named _fullfile_
    load('filelist_test.mat') %%% loads filelist in cell array _fullfile_
    load('decisions_test_h#_pau_epi.mat') %%% loads _files_, which contains starts and stops for non-speech parts
    if filesNeeded > length(fullfile)
        error('Not enough speech files!');
    end

    randomFiles = randperm(length(fullfile),filesNeeded);

    % fid = fopen('D:\Skole\Prosjektoppgave (TTT4560)\nospeech\drysrc\ir383.dat','r'); imp = fread(fid, 'float'); fclose(fid);
    % pw = sum(imp.^2); %%% power estimate of impulse response
    % imp = imp/sqrt(pw); %%% impule response of a meeting room
    
    fs = 8000; %%% desired sample rate
    
    %%% READ SPEECH FILES FROM TIMIT:
    speechFiles = fullfile(randomFiles); %%% list of file name-strings of speech files to be used
    speech = cell(size(speechFiles)); %%% empty cell matrix to store tranisents

    for l = 1:length(speechFiles)
        [speech{l}.orgdata, speech{l}.orgfs] = audioread(['D:\Skole\Prosjektoppgave (TTT4560)\Bendik\Databases\TIMIT\train_wav\', speechFiles{l}, '.wav']);
        fsSpeech = speech{l}.orgfs; %%% %%% sample rate for native TIMIT. Same f_s for all speech files
        dec = speech{l}.orgfs/fs; %%% find decimation factor
        speech{l}.data = decimate(speech{l}.orgdata, dec, 'FIR'); %%% decimate by factor _dec_
    %     tmp = decimate(speech{l}.orgdata, dec, 'FIR'); %%% decimate by factor _dec_
    %     norm = sqrt(sum(tmp.^2)); %%% find signal power estimate
    %     speech{l}.data = tmp/norm; %%% normalize signal and store in struct-field _data_
        clear tmp norm
    end
    %%% /READ SPEECH FILES FROM TIMIT

    %%% CREATE TRANSIENT SOUNDS:
    fsTrans = 16000; %%% desired sample rate for transients (same as native TIMIT)
    %%% all available files: %transientFiles = {'aircap', 'buzzer', 'candybwl', 'cap1', 'china2', 'clap1', 'coin2', 'colacan', 'dice1', 'metal05', 'phone4', 'pipong', 'stapler', 'teak3', 'tear'};
    transientFiles = {'aircap', 'cap1', 'china2', 'clap1', 'colacan', 'metal05', 'teak3'}; %%% most suiting files chosen
    noTransients = length(transientFiles);
    transient = cell(size(transientFiles)); %%% empty cell array to store tranisents

    totLength = 0;
    for k = 1:noTransients
        [transient{k}.orgdata, transient{k}.orgfs] = audioread(['D:\Skole\Prosjektoppgave (TTT4560)\nospeech\drysrc\', transientFiles{k}, '.wav']);
        transient{k}.name = transientFiles{k}; %%% store descriptive name for transient sound
        %%%tmp = conv(transient{k}.orgdata, imp); %%% convolves dry sound with impulse response of a meeting room
        decTrans = transient{k}.orgfs/fsTrans; %%% find decimation factor
        tmp2 = decimate(transient{k}.orgdata, decTrans, 'FIR'); %%% decimate by factor _dec_
        norm = sqrt(sum(tmp2.^2)); %%% find signal power estimate
        transient{k}.data = tmp2/norm; %%% normalize signal and store in struct-field _data_
        %%%transient{k} = setfield(transient{k}, 'data', tmp2/norm);
        totLength = totLength + length(transient{k}.data); %%% accumulate length of transients
        clear tmp tmp2 norm
    end
    avgLength = totLength/noTransients;
    avgLengthFrames = avgLength/(2*stpsz); %%% approx. average transient length in frame increments (shifts) !!! (@ fs = 8000) !!!
    %%% /CREATE TRANSIENT SOUNDS

    %%%(not needed) annotation = files(randomFiles); %%% cell array of the starts and stops
    testFiles = cell(1,noFilesTot);

    structureOfPauses = [0 0 1 0 1 0 1 0]; %%% indicates the location of pauses (1) and speech segments (0). Will be randomly permuted.
    noSegs = length(structureOfPauses);

    %%% CONCATENATE FILES
    for m = 1:noFilesTot
        x = [];
        starts = [];
        stops = [];
        segLengths = [];
        shifts = [];
        offset = 0;
        %%% DO NOT ALLOW INITIAL PAUSE:
        firstSegIsNotPause = 0; %%% to start the following while-loop
        while firstSegIsNotPause == 0 %%% do not allow first segment to be a pause segment
            pau = structureOfPauses(randperm(noSegs)); %%% randomly permutes the pause/speech concatenation structure
            if pau(1) == 1
                firstSegIsNotPause = 0;
            else
                firstSegIsNotPause = 1;
            end
        end
        %%% /DO NOT ALLOW INITIAL PAUSE
        %totLength = 0;
        pauses = randi(pauseLength, 1, sum(structureOfPauses)); %%% yields sum(structureOfPauses) random pause lengths
        o = 1; %%% keeps track of "pause number"
        s = 1; %%% counter for which speech segment is to be used
        for q = 1:noSegs %%% [m*n is the correct index for the random files]
            %totLength = totLength + length(speech{m*n}.orgdata);
            if pau(q) %%% current segment is to be inserted silence %%% [place silence before these speech segments]
                x = [x; zeros(fsSpeech*pauses(o),1)];
                starts = [starts, offset];
                stops = [stops, fsSpeech*pauses(o) + offset];
                segLengths = [segLengths, fsSpeech*pauses(o)];
                shifts = [shifts, offset];
                offset = offset + fsSpeech*pauses(o);
                o = o + 1;
            else %%% current segment is to be speech
                x = [x; speech{m*s}.orgdata];
                starts = [starts, files{randomFiles(m*s)}.start + offset];
                stops = [stops, files{randomFiles(m*s)}.stop + offset];
                segLengths = [segLengths, length(speech{m*s}.orgdata)];
                shifts = [shifts, offset]; %%% should be updated before _offset_
                offset = offset + length(speech{m*s}.orgdata); %%% offset for start of next segment
                s = s + 1;
            end
        end
        testFiles{m}.concatenated = x;
        if issorted(starts) %%% makes sure starts are properly sorted in ascending order
            testFiles{m}.starts = starts;
        else
            error(['Starting points are not in ascending order for file no. ' num2str(m)]);
        end
        if issorted(stops) %%% makes sure stops are properly sorted in ascending order
            testFiles{m}.stops = stops;
        else
            error(['Stoping points are not in ascending order for file no. ' num2str(m)]);
        end
        testFiles{m}.pauses = pauses;
        testFiles{m}.segments = segLengths;
        testFiles{m}.shifts = shifts;
        testFiles{m}.structure = pau;
    end
    %%% /CONCATENATE FILES

    %%% ADD TRANSIENTS
    numTrans = 5; %%% number of transients to be added to each file
    %scaling = ones(1,numTrans); %%% scaling for transient sounds
    scalingInit = [1 1.5 2 3 3]; %%% user specified scaling of transients
    for t = 1:noFilesTot
    	scaling = scalingInit;
        tranStart = zeros(1,numTrans);
        tranStop = zeros(1,numTrans);
        randomTrans = randi(length(transient), 1, numTrans); %%% draws _numTrans_ transients to be inserted
        lenTrans = zeros(1,numTrans);
        taken = [];
        for i = 1:numTrans
            lenTrans(i) = length(transient{randomTrans(i)}.data);
            maxStart = length(testFiles{t}.concatenated) - lenTrans(i); %%% last possible starting point without cutting away anything from transient
            randStart = randi(maxStart,1,1); %%% random starting point for transient
            errCnt = 0; %%% tracks number of iterations (to prevent infinite loop)
            while ~isempty(intersect(taken,randStart:(randStart+lenTrans(i)-1))) || errCnt < 20
                randStart = randi(maxStart,1,1); %%% generate new starting point for transient. repeated until non-overlapping transients or 20 tries
                errCnt = errCnt + 1;
            end
            tranStart(i) = randStart;
            tranStop(i) = randStart + lenTrans(i) - 1;
            %%% CLIPPING PREVENTION:
            %%% MAKE SURE ADDITION OF TRANSIENTS NOT RESULT IN CLIPPING (AMPLITUDES OVER 1).
            maxAmplitudeAllowed = 1; %%% determines maximum allowed amplitude of signal
            tempA = testFiles{t}.concatenated(tranStart(i):tranStop(i)); %%% for more readable code
            tempB = transient{randomTrans(i)}.data; %%% for more readable code
            tempSig = tempA + scaling(i).*tempB; %%% suggested signal segment (contains possibly clipping)
            tempScale = scaling(i); %%% so that _tempScale_ is defined (to original scaling) if no clipping occurs
            pos = find(abs(tempSig) > maxAmplitudeAllowed, 1, 'first'); %%% find first clipping occurence, if any
            while ~isempty(pos) %%% do while any samples > 1
                tempScale = (maxAmplitudeAllowed-abs(tempA(pos)))/(abs(tempB(pos)));  %%% calculate new scaling-coefficiet
                                                                            %%% a(n) + scale*b(n) <= maxAmplitude => scale = (maxAmplitude-a(n))/b(n)
                tempSig = tempA + tempScale.*tempB; %%% create new section with transient
                pos = find(tempSig > maxAmplitudeAllowed); %%% check for remaining clipping
            end
            scaling(i) = tempScale; %%% replace inital scaling with new one
            clear tempA tempB tempSig tempScale pos
            %%% /CLIPPING PREVENTION
            testFiles{t}.concatenated(tranStart(i):tranStop(i)) = testFiles{t}.concatenated(tranStart(i):tranStop(i)) + scaling(i).*transient{randomTrans(i)}.data; %%% add transient
            taken = [taken, tranStart(i):tranStop(i)]; %%% keeps track of where transients already are placed
        end
        testFiles{t}.tranStart = tranStart;
        testFiles{t}.tranStop = tranStop;
        testFiles{t}.randomTrans = randomTrans;
        testFiles{t}.fs = fsSpeech;
        testFiles{t}.transientScaling = scaling;
        for n = 1:numTrans
            testFiles{t}.transientNames{n} = transientFiles{randomTrans(n)}; %%% transient names
        end
    end
    %%% /ADD TRANSIENTS
    
    tranAvgLen = avgLengthFrames;
    
    answer = input('Save files and overwrite _testFiles.mat_? y/n\n', 's')
    if strcmp(answer, 'y')
        save('testFiles.mat', 'testFiles', 'tranAvgLen');
    end
    
    %%% PLOT THE RESULTING FILES WITH ANNOTATION
    if plotFiles
        for ii = 1:noFilesTot
            if ((mod((ii-1), 2)+1) == 1) %%% plot two files in each figure window
                figure();
            end
            subplot(2,1,(mod((ii-1), 2)+1))
            con = plot(testFiles{ii}.concatenated);
            hold on;
            %%% plot segment starts
            for i = 1:length(testFiles{ii}.shifts) %%% same length for all files
                seg(i) = plot([testFiles{ii}.shifts(i)+1, testFiles{ii}.shifts(i)+1],[-0.3 0.3], 'r:', 'LineWidth', 2);
                ylim([-0.5 0.5]);
            end
            for i = 1:length(testFiles{ii}.starts)
                str(i) = plot([testFiles{ii}.starts(i), testFiles{ii}.starts(i)],[-0.3 0.3], 'b--');
                ylim([-0.5 0.5]);
            end
            for i = 1:length(testFiles{ii}.stops)
                stp(i) = plot([testFiles{ii}.stops(i), testFiles{ii}.stops(i)],[-0.3 0.3], 'g--');
                ylim([-0.5 0.5]);
            end
            for i = 1:length(testFiles{ii}.tranStart)
                trstr(i) = plot([testFiles{ii}.tranStart(i), testFiles{ii}.tranStart(i)],[-0.3 0.3], 'c-');
                ylim([-0.5 0.5]);
            end
            for i = 1:length(testFiles{ii}.tranStop)
                trstp(i) = plot([testFiles{ii}.tranStop(i), testFiles{ii}.tranStop(i)],[-0.3 0.3], 'c:');
                ylim([-0.5 0.5]);
            end
            legend([seg(1) con str(1) stp(1) trstr(1) trstp(1)], 'Segment starts', 'Concatenated files', 'Silence start', 'Silence stop', 'Transient start','Transient stop')
        end
    end
    %%% /PLOT THE RESULTING FILES WITH ANNOTATION
end