function bdy=deci_bdy(mu,vari,wgt, varargin)

%deci_bdy - deciding the boundary of one subband based on the GMM
% The parameters in this function is designed for signals with sampling
% rate 4k Hz.
% Inputs:
%   sig     the waveform of the input signal
%   thld    the normalized voting threshold
%   speech  the reference clean speech signal
%
% Outputs:
%   outvad    the vad labels, 1 for speech and 0 for nonspeech
%   spp       speech presence probability in the time-frequency domain
%
% reference paper: D. Ying, Y. Yan, J. Dang, and F. Soong, "Voice activity detection based on unsupervised learning
% framework," IEEE Trans. Audio, Speech and Language Processing, 19(8), Nov,
% 2011
%June-15
%Dongwen Ying

%%% ADDED 2014.05.31:
if ~isempty(varargin)
    deciGamma = varargin{1}; %%% SUPPORTS GAMMA AS INPUT
else
    deciGamma = 0.45; %%% SETS GAMMA AS IT ORIGINALLY WAS
end
%%% /ADDED 2014.05.31

a = vari(1)- vari(2);
b = 2*mu(1)*vari(2)-2*mu(2)*vari(1); 
%c = mu(2)*mu(2)*vari(1)-mu(1)*mu(1)*vari(2)-vari(1)*vari(2)*log(vari(1)*wgt(2)/vari(2)/wgt(1)); % original
c = mu(2)*mu(2)*vari(1)-mu(1)*mu(1)*vari(2)-vari(1)*vari(2)*log(vari(1)*wgt(2)^2/vari(2)/wgt(1)^2); % correct
    
bdy1 = (-b+sqrt(b^2-4*a*c))/(2*a);
bdy2 = (-b-sqrt(b^2-4*a*c))/(2*a);   

if((bdy1-mu(1))*(bdy1-mu(2))<0)
    bdy=bdy1;
else
    if((bdy2-mu(1))*(bdy2-mu(2))<0)
        bdy=bdy2;
    else
        bdy=max(bdy1,bdy2);
    end
end 

% This comes from eq. 24 in the paper.
%%% CONSIDER CHANGING THIS FOR IMPROVED PERFORMANCE:
bdy = min(mu) + deciGamma*(bdy-min(mu));  %%%0.45 for 8 channels
%%% /CONSIDER CHANGING THIS FOR IMPROVED PERFORMANCE
