function  [outvad, z, mu_veri, vari_veri, thres]= gmmVad(sig, thld, threshold, tranAvgLen, deciGamma)

global TRANSIENT FEATURE
%%
% % % if nargin == 3
% % %     threshold = varargin{1};
% % % end

%GMMVAD - voice activity detection by using a sequential Gaussian mixture
%model
% The parameters in this function is designed for signals with sampling
% rate 4k Hz.
% Inputs:
%   sig     the waveform of the input signal
%   thld    the normalized voting threshold
%   speech  the reference clean speech signal
%
% Outputs:
%   outvad    the vad labels, 1 for speech and 0 for nonspeech
%   spp       speech presence probability in the time-frequency domain
%
% reference paper: D. Ying, Y. Yan, J. Dang, and F. Soong, "Voice activity detection based on unsupervised learning
% framework," IEEE Trans. Audio, Speech and Language Processing, 19(8), Nov,
% 2011
%June-15
%Dongwen Ying

%%
theta = 3.5;
num_chnl = 8; % number of subbands used

framesz = 160; stpsz = 80; % frame size 20ms, stepsize 10ms
fftsz = 256;
alpha = 0.99;
M = 60;  % number of frames used to initialize the GMM
SRate = 8000; % sample rate

lowfreq = 0;
NFilter = 8; % number of bands in the filterbanks
MelMax = 2595*log10(1 + (SRate/2.0-lowfreq)/700.0);
MelStp = MelMax / NFilter;
Freq = lowfreq + 700*(exp( (0:NFilter)*MelStp / 2595*log(10) ) - 1.0);
IndArray = fix(Freq * fftsz / SRate);

num = fix((length(sig)-framesz+stpsz)/stpsz); % number of frames in the input

idx=zeros(num,num_chnl);

[xmel,mn,mx]=melbankm(num_chnl,fftsz,SRate,0.000,0.5,'t'); % triangular shaped filters in mel domain

%% added 2014.03.13-14
mu_veri = zeros(num_chnl, 2*num); %%% for testing purposes
vari_veri = zeros(num_chnl, 2*num); %%% for testing purposes
wgt_veri = zeros(num_chnl, 2*num); %%% for testing purposes
%% /added 2014.03.13-14


% calculate the log-energy for each subband of the entire input signal
ind = 1 : framesz;
for cnt = 1 : num
    fr = sig(ind); % extract one frame at a time
    ind = ind + stpsz;
    f = abs(fft(fr, fftsz));
    f = f(1 : fftsz / 2);
    spec(:,cnt) = f(:);
    f = f; % why?
    rtx = (f(mn : mx)).*conj(f(mn : mx)); % same as |f(mn:mx)|²
    z(:, cnt) = 10.* log10(xmel * rtx); % calculate the log-energy in each subband
end

for jj = 1 : num_chnl % one melband at a time
    clear ctur;
    
    x=z(jj,:); % one melband of the entire file.
    
    % median filter the energy signal for each mel band(num)
    ctur = zeros(1, num); %%% pre-allocate for speed, length(x) = num
    for cnt=1:num
        ins = max(cnt-4,1):cnt; %%% four previous frames and current (no delay needed)
        ctur(cnt) = mdfilter2(x(ins)); %%% median filtered log-energies are 
        
        %%% PREVIOUSLY:
        %%%ins = max(cnt-2,1):min(cnt+2,num);
        %%%ctur(cnt) = mdfilter(x(ins),cnt); % Smoothed energy envelope
        %%% /PREVIOUSLY
    end
    
    z(jj, :) = ctur;
    
    [mu,vari,wgt] = iniGMM(ctur(1:M),theta); % initialize the GMM by using the M first samples of the smoothed energy envelope.
    if mu(1)>mu(2)
        mu = fliplr(mu);
        vari = fliplr(vari);
        wgt = fliplr(wgt);
    end
    
    for cnt=1:M
        curprob = 0.3989.*wgt./sqrt(vari).*exp(-0.5.*(ctur(cnt)-mu).^2./vari);
        p = curprob./sum(curprob);
        probs_1(jj,cnt) = curprob(1);
        probs_2(jj,cnt) = curprob(2);
    end
    
    bdy(1:M) = deci_bdy(mu,vari,wgt, deciGamma); % one commom treshold is chosen for the first 60 frames.
    % the threshold is made on the base from the GMM initialization.
    
    %% added 2014.03.13-14
    mu_veri(jj,1:2:2*M) = mu(1); %%% non-speech
    mu_veri(jj,2:2:2*M) = mu(2); %%% speech
    vari_veri(jj,1:2:2*M) = vari(1); %%% non-speech
    vari_veri(jj,2:2:2*M) = vari(2); %%% speech
    wgt_veri(jj,1:2:2*M) = wgt(1); %%% non-speech
    wgt_veri(jj,2:2:2*M) = wgt(2); %%% speech
    %% /added 2014.03.13-14
    
    %%%%%%%%%%%%%%%%%% added  %%%%%%%%%%%%%%%%%%%%%%%%%%
    thres(1:M, jj) = bdy(1:M);
    %%%%%%%%%%%%%%%%%% added  %%%%%%%%%%%%%%%%%%%%%%%%%%
    
    
    pos = find(ctur(1:M)>bdy(1));
    idx(pos,jj)=1; % classify the first 60 frames
    
    %%%%%%%%%%%%%%%%%% added  %%%%%%%%%%%%%%%%%%%%%%%%%%
    n_level(1:M, jj) = min(mu); % noise level
    s_level(1:M, jj) = max(mu); % speech level
    min_vari(1:M, jj) = min(vari);
    max_vari(1:M, jj) = max(vari);
    w_1(1:M, jj) = wgt(1);
    w_2(1:M, jj) = wgt(2);
    %%%%%%%%%%%%%%%%%% added  %%%%%%%%%%%%%%%%%%%%%%%%%%
    
    for cnt=M+1:num % rest of the speech file in the same subband.
        y = ctur(cnt);
        
        % eq. 12
        curprob = 0.3989.*wgt./sqrt(vari).*exp(-0.5.*(y-mu).^2./vari);
        p = curprob./sum(curprob);
        
        wgt_new = alpha.*wgt + (1-alpha).*p; % eq. 15
        mu_new  = (alpha.*wgt.*mu + (1-alpha)*y.*p) ./ wgt_new; % eq. 17
        mu_new(2) = max(mu_new(2), mu_new(1)+theta); % constraint
        vari_new  = (alpha.*wgt.*vari + (1-alpha).*p.*(y-mu_new).^2) ./ wgt_new; % eq. 21
        vari_new(2) = max(vari_new(2), 1.1*vari_new(1)); % constraint original
        %vari_new(2) = max(vari_new(2), vari_new(1)); % constraint changed
        
        %%%%%%%%%%%%%%%%%% added constraint %%%%%%%%%%%%%%%%%%%%%%%%%%
        %wgt_new(2) = max(wgt_new(2), 0.03);
        %wgt_new(1) = 1 - wgt_new(2);
        %%%%%%%%%%%%%%%%%% added constraint %%%%%%%%%%%%%%%%%%%%%%%%%%
        
        vari = vari_new;
        wgt = wgt_new;
        mu = mu_new;
        
        %% added 2014.03.13-14
        mu_veri(jj, (2*cnt-1):(2*cnt)) = mu;
        vari_veri(jj, (2*cnt-1):(2*cnt)) = vari;
        wgt_veri(jj, (2*cnt-1):(2*cnt)) = wgt;
        %% /added 2014.03.13-14
        
        if (cnt == 578) && (jj == 6)
            if length(bdy(imag(bdy) == 0)) < num -1
                %disp('complex number')
            end
        end
        
        bdy(cnt) = deci_bdy(mu,vari,wgt, deciGamma);
        
        %%%%%%%%%%%%%%%%%% added  %%%%%%%%%%%%%%%%%%%%%%%%%%
        thres(cnt, jj) = bdy(cnt);
        n_level(cnt, jj) = min(mu); % noise level
        s_level(cnt, jj) = max(mu); % speech level
        min_vari(cnt, jj) = min(vari);
        max_vari(cnt, jj) = max(vari);
        w_1(cnt, jj) = wgt(1);
        w_2(cnt, jj) = wgt(2);
        %%%%%%%%%%%%%%%%%% added  %%%%%%%%%%%%%%%%%%%%%%%%%%
        
        probs_1(jj,cnt) = curprob(1);
        probs_2(jj,cnt) = curprob(2);
        probs(cnt) = p(2); % probability for speech stored to plot at end
        
        if bdy(cnt) < y
            idx(cnt,jj)=1;
        end
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%plot%%%%%%%%%%%%%%%%%%
%     if nargin>2
%         subplot('Position',[0.08 0.4 0.92 0.26]);
%         plot(bdy)
%         hold on;
%         plot(ctur,'r')
%         hold on;
%         plot(n_level,'g')
%         hold on;
%         plot(s_level,'k')
%         hold on;function  [outvad z]= gmmVad(sig,thld,speech)


outids = sum(idx,2)./num_chnl;

pos = find(outids>=thld);
idx_new = zeros(1,length(idx)); % NOT ORIGINAL
idx_new(pos) = 1; % idx originally
%idx(pos) = 1;

%%%hangover scheme from ARM
bth=4;
hth=5;%hth=7;
bc=0;
hc=0;
for j=1:num
    if idx_new(j)==1 % idx originally
        %if idx(j) == 1
        outvad(j)=1;
        bc=bc+1;
        if bc>bth
            hc=hth;
        end
    else
        bc=0;
        hc=hc-1;
        if hc<=0
            outvad(j)=0;
            hc=0;
        else
            outvad(j)=1;
        end
    end
end


if TRANSIENT
%%%    if ~FEATURE
        [outvad ratio] = transient_detection(probs_1, probs_2, outvad, threshold, tranAvgLen);
%%%    else
%%%        [outvad ratio] = transient_detection(probs_1, probs_2, outvad);
%%%    end
end
