function [outvad, z, mu_veri, vari_veri, bdy, cdfThreshold, mu_trans_veri, vari_trans_veri, wgt_trans_veri] = hlpVADupd(sig, num_chnl, framesz, stpsz, theta, fftsz, alpha, thld, M, varargin)
% R = frameVAD(...)
global VADDECIDEUPDATE SPF

%%% ADDED 2014.05.12:
variFreeze = 0; %%% 1: freeze variance and mean; 0 freeze only mean
wgtFreeze = 0; %%% 1: freeze wgt and mean; 0 freeze only mean
%%% /ADDED 2014.05.12

%%% ADDED 2014.06.01:
vadDecideUpdate = VADDECIDEUPDATE;
%%% /ADDED 2014.06.01

speech_prob_floor = SPF;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% UPDATED VERSION TO INVESTIGATE NEW MODEL FREEZE %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Do VAD as the orig. GMM VAD, but freeze a copy  %%%
%%% of the model parameters for utilisation in      %%%
%%% the transient detection.            2014.04.29  %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% NB! Transients are not detected in the _M_ first frames.

%     global SNR NIST WHITE_NOISE GAMMA
%     SNR = 30;
%     NIST = 1;
%     WHITE_NOISE = 1;
%     GAMMA = 0;

%%% FOR TRANSIENT DETECTION
    speechFlag = 1; %%% the flag used to denote speech in _outvad_
    trsp = -1; %%% denotes transient in speech. Consistent with _creatFrameAnnotation_ notation.
    trnsp = -2; %%% denotes transient in non-speech; Consistent with _creatFrameAnnotation_ notation.
    if length(varargin) > 2
        transientLength = round(varargin{3});
    else
        transientLength = 80; %%% hard coded length of transient in number of frames (80 is ~avg. length)
    end
%%% /FOR TRANSIENT DETECTION

    if length(varargin) == 4
        deciGamma = varargin{4};
    else
        deciGamma = 0.45;
    end
    
    avg = 1; %%% use moving average filtering instead of median filtering
    
    if (isempty(varargin))
        weights = [1 1 1 1 1 1 1 1]; %%% _weights_ controls how each sub-band is weighted in the VAD decision
    else
        weights = varargin{1};
        if length(varargin) >= 2
            transThresh = varargin{2}; %%% threshold for the transient detection. see below
        else
            transThresh = 0.98; %%% based on what was found in Paulsrud, 2013
        end
    end
    
%     if avg
%         speech_prob_floor = 0.25;
%     else
%         speech_prob_floor = 0.40;
%     end

    %w = [1 1 1 1 1 1 1 1];
    w = [10 40 50 45 35 15 5 5]; %%% _w_ is used to weigh each sub-band in the frame update decision
    w = w'./sum(w);
    
    if (length(sig) < framesz)
        error('Too little input data');
    end
    
    %%%M = 60;  % number of frames used to initialize the GMM
    SRate = 8000; % sample rate

    %lowfreq = 0;
    %%%NFilter = num_chnl; % number of bands in the filterbanks
    %%%MelMax = 2595*log10(1 + (SRate/2.0-lowfreq)/700.0);
    %%%MelStp = MelMax / NFilter;
    %%%Freq = lowfreq + 700*(exp( (0:NFilter)*MelStp / 2595*log(10) ) - 1.0);
    %%%IndArray = fix(Freq * fftsz / SRate);
    
    num = fix((length(sig)-framesz+stpsz)/stpsz); % number of frames in the input
    
    [xmel,mn,mx]=melbankm(num_chnl,fftsz,SRate,0.000,0.5,'t'); % triangular shaped filters in mel domain
    
    z_tmp = zeros(num_chnl, num); %%% pre-declared for speed
    z = zeros(num_chnl, num); %%% pre-declared for speed
    
    idx=zeros(num,num_chnl); %%% [frames x channels] speech flags for every band in every frame
    idx_out=zeros(num,num_chnl); %%% [frames x channels] speech flags for every band in every frame
    idx_new = zeros(1,num); % decision for each frame
    ws = zeros(1, num);
    mu_veri = zeros(num_chnl, 2*num); %%% for testing purposes
    vari_veri = zeros(num_chnl, 2*num); %%% for testing purposes
    wgt_veri = zeros(num_chnl, 2*num); %%% for testing purposes
    mu_frame_veri = zeros(num_chnl, 2*num); %%% for testing purposes
    vari_frame_veri = zeros(num_chnl, 2*num); %%% for testing purposes
    wgt_frame_veri = zeros(num_chnl, 2*num); %%% for testing purposes
    
    mu = zeros(num_chnl, 2); %%% pre-declared for speed. 
    vari = zeros(num_chnl, 2); %%% pre-declared for speed.
    wgt = zeros(num_chnl, 2); %%% pre-declared for speed.
    
    %%% TRANSIENT COPY:
    mu_trans = zeros(num_chnl, 2); %%% pre-declared for speed. 
    vari_trans = zeros(num_chnl, 2); %%% pre-declared for speed.
    wgt_trans = zeros(num_chnl, 2); %%% pre-declared for speed.
    mu_trans_veri = zeros(num_chnl, 2*num); %%% for testing purposes
    vari_trans_veri = zeros(num_chnl, 2*num); %%% for testing purposes
    wgt_trans_veri = zeros(num_chnl, 2*num); %%% for testing purposes
    probs_1_trans = zeros(num_chnl, num); %%% pre-declared for speed (NB! the dimensions are the opposite of other variables
    probs_2_trans = zeros(num_chnl, num); %%% pre-declared for speed (NB! the dimensions are the opposite of other variables
    %%% /TRANSIENT COPY
    
    cdfThreshold = zeros(1,num); %%% the threshold are stored only for verification/testing purposes
    
    bdy = zeros(num, num_chnl); %%% pre-declared for speed
    thres = zeros(num, num_chnl); %%% pre-declared for speed
    n_level = zeros(num, num_chnl); %%% pre-declared for speed
    s_level = zeros(num, num_chnl); %%% pre-declared for speed
    min_vari = zeros(num, num_chnl); %%% pre-declared for speed
    max_vari = zeros(num, num_chnl); %%% pre-declared for speed
    w_1 = zeros(num, num_chnl); %%% pre-declared for speed
    w_2 = zeros(num, num_chnl); %%% pre-declared for speed
    probs = zeros(2, 2*num); %%% pre-declared for speed
    probs_1 = zeros(num_chnl, num); %%% pre-declared for speed (NB! the dimensions are the opposite of other variables
    probs_2 = zeros(num_chnl, num); %%% pre-declared for speed (NB! the dimensions are the opposite of other variables
    
    ratio = zeros(1,num); %%% pre-declared for speed
    transientRemainder = 0; %%% initialization
    
    %%%hangover scheme from ARM
    outvad = zeros(1, num); %%% pre-declare for speed
    bth=4;
    hth=5;%hth=7;
    bc=0;
    hc=0;
    
    %%%
%     load('initializationParametersWhiteNoise.mat'); %%% loads variables _mu_prior_, _vari_prior_ and _wgt_prior_   
%     mu = mu_prior;
%     vari = vari_prior;
%     wgt = wgt_prior;
    %%%
    
    ind = 1 : framesz; %%% indices of the current frame, first frame
    
    
    %%%
    for i = 1:num
        fr = sig(ind); % extract one frame at a time
        ind = ind + stpsz; %%% update current frame indices
        f = abs(fft(fr, fftsz)); %%% Magnitude response of current frame
        f = f(1 : fftsz / 2); %%% Extract 0 -> 0.5 of normalized frequenzy
        rtx = (f(mn : mx)).*conj(f(mn : mx)); % same as |f(mn:mx)|² %%% squared magnitude of signal at non-zero mel-coeficients
        z_tmp(:, i) = 10.*log10(xmel * rtx); % calculate the log-energy in each subband %%% stores log-energy per frame (as columns) for each sub-band (as rows)
        
        mu_frame = zeros(num_chnl, 2); %%% to store suggested modelupdates for current frame. Reset for each frame.
        vari_frame = zeros(num_chnl, 2); %%% to store suggested modelupdates for current frame. Reset for each frame.
        wgt_frame = zeros(num_chnl, 2); %%% to store suggested modelupdates for current frame. Reset for each frame.
        
        if (i < M) %%% FOR FRAMES <M:
            for j = 1:num_chnl
                ins = max(i-4,1):i; %%% four previous frames and current (no delay needed)
                if avg
                    z(j, i) = mdfilter2(z_tmp(j,ins)); %%% moving average filtered log-energies 
                else
                    z(j, i) = median(z_tmp(j,ins)); %%% median filtered log-energies
                end
                %%%ins = max(i-2,1):min(i+2,num); %%% five "current" indices
                %%%z(j, i) = mdfilter(z_tmp(j,ins), i); %%% median filtered log-energies are 
            end
            continue %%% let the VAD buffer M frames before doing anything
        elseif (i == M) %%% FOR FRAME M:
            for j = 1:num_chnl
                ins = max(i-4,1):i; %%% four previous frames and current (no delay needed)
                if avg
                    z(j, i) = mdfilter2(z_tmp(j,ins)); %%%  %%% moving average filtered log-energies
                else
                    z(j, i) = median(z_tmp(j,ins)); %%% median filtered log-energies
                end
                %%%ins = max(i-2,1):min(i+2,num); %%% five "current" indices
                %%%z(j, i) = mdfilter(z_tmp(j,ins), i); %%% median filtered log-energies are 
                
                %%% Initialize model parameters
                [mu(j,:),vari(j,:),wgt(j,:)] = iniGMM(z(j,1:M),theta); %%% initialize the GMM by using the M first frames of the smoothed energy envelope.
                
                if mu(1)>mu(2) %%% if (should not be possible) the model components are returned in [speech | non-speech] format
                    mu(j,:) = fliplr(mu(j,:));
                    vari(j,:) = fliplr(vari(j,:));
                    wgt(j,:) = fliplr(wgt(j,:));
                end
                
                %%%% TRANSIENT COPY:
                mu_trans(j,:) = mu(j,:);
                vari_trans(j,:) = vari(j,:);
                wgt_trans(j,:) = wgt(j,:);
                %%%% /TRANSIENT COPY:
                
                bdy(1:M,j) = deci_bdy(mu(j,:),vari(j,:),wgt(j,:), deciGamma); %%% one (per sub-band) commom treshold is chosen for the first 60 frames
                
                for cnt=1:M
                    y = z(j,cnt); %%% extract current frame's log-energy contour
                    curprob = 0.3989.*wgt(j,:)./sqrt(vari(j,:)).*exp(-0.5.*(y-mu(j,:)).^2./vari(j,:));
                    p = curprob./sum(curprob);
                    probs_1(j,cnt) = curprob(1);
                    probs_2(j,cnt) = curprob(2);
                    %%% TRANSIENT COPY:
                    probs_1_trans(j,cnt) = curprob(1);
                    probs_2_trans(j,cnt) = curprob(2);
                    %%% /TRANSIENT COPY
                    probs(j,(2*cnt-1)) = p(1); % probability for speech stored to plot at end
                    probs(j,(2*cnt)) = p(2); % probability for speech stored to plot at end
                    
                    if bdy(M, j) < y %%% bdy(1,j) == bdy(2,j) == ... == bdy(M - 1,j) == bdy(M,j)
                        idx(cnt,j)=1; %%% classification for first M frames
                    end
                    
                    outid = sum(idx(cnt,:)*weights')/sum(weights); %%% sum over each channel, weighted.

                    if outid>=thld
                        idx_new(cnt) = 1; % idx originally
                    end

                    %%%hangover scheme from ARM
                    if idx_new(cnt)==1
                        outvad(cnt)=1;
                        bc=bc+1;
                        if bc>bth
                            hc=hth;
                        end
                    else
                        bc=0;
                        hc=hc-1;
                        if hc<=0
                            outvad(cnt)=0;
                            hc=0;
                        else
                            outvad(cnt)=1;
                        end
                    end
                end  
                
                %%%%%%%%%%%%%%%%%% added  %%%%%%%%%%%%%%%%%%%%%%%%%%
                thres(1:M, j) = bdy(1:M, j);    %%% storage variables
                n_level(1:M, j) = min(mu(j,:)); % noise level
                s_level(1:M, j) = max(mu(j,:)); % speech level
                min_vari(1:M, j) = min(vari(j,:));
                max_vari(1:M, j) = max(vari(j,:));
                w_1(1:M, j) = wgt(1);
                w_2(1:M, j) = wgt(2);
                %%%%%%%%%%%%%%%%%% added  %%%%%%%%%%%%%%%%%%%%%%%%%%

                %probs_1(j,1:M) = curprob(1);
                %probs_2(j,1:M) = curprob(2);
                %probs(j,1:2:2*M) = p(1); % probability for speech stored to plot at end
                %probs(j,2:2:2*M) = p(2); % probability for speech stored to plot at end
            end
        else %%% FOR FRAMES M:END:
            for j = 1:num_chnl
                ins = max(i-4,1):i; %%% four previous frames and current (no delay needed)
                if avg
                    z(j, i) = mdfilter2(z_tmp(j,ins));  %%% moving average filtered log-energies
                else
                    z(j, i) = median(z_tmp(j,ins)); %%% median filtered log-energies
                end
                %%%ins = max(i-2,1):min(i+2,num); %%% "five" "current" indices
                %%%z(j, i) = mdfilter(z_tmp(j,ins), i); %%% median filtered log-energies are 

                y = z(j,i); %%% extract current frame's log-energy contour

                curprob = 0.3989.*wgt(j,:)./sqrt(vari(j,:)).*exp(-0.5.*(y-mu(j,:)).^2./vari(j,:)); %%% eq. 12, stp. 1
                %%% TRANSIENT COPY:
                curprob_trans = 0.3989.*wgt_trans(j,:)./sqrt(vari_trans(j,:)).*exp(-0.5.*(y-mu_trans(j,:)).^2./vari_trans(j,:)); %%% eq. 12, stp. 1
                %%% /TRANSIENT COPY
                p = curprob./sum(curprob); %%% eq. 12, stp. 1

                wgt_new = alpha.*wgt(j,:) + (1-alpha).*p; % eq. 15 %%% model update, alpha determines how much new frames are weigthed
                mu_new  = (alpha.*wgt(j,:).*mu(j,:) + (1-alpha)*y.*p) ./ wgt_new; % eq. 17
                mu_new(2) = max(mu_new(2), mu_new(1)+theta); % constraint
                vari_new = (alpha.*wgt(j,:).*vari(j,:) + (1-alpha).*p.*(y-mu_new).^2) ./ wgt_new; % eq. 21
                vari_new(2) = max(vari_new(2), 1.1*vari_new(1)); % constraint original

                mu_veri(j, (2*i-1):(2*i)) = mu(j,:); %%% store means (non-speech | speech) for verification purposes (only for testing)
                vari_veri(j, (2*i-1):(2*i)) = vari(j,:); %%% store variances (non-speech | speech) for verification purposes (only for testing)
                wgt_veri(j, (2*i-1):(2*i)) = wgt(j,:); %%% store weights (non-speech | speech) for verification purposes (only for testing)

                mu_frame(j,:) = mu_new;
                vari_frame(j,:) = vari_new;
                wgt_frame(j,:) = wgt_new;
                
                %%%%%%%
                probs_1(j,i) = curprob(1);
                probs_2(j,i) = curprob(2);
                %%% TRANSIENt COPY:
                probs_1_trans(j,i) = curprob_trans(1);
                probs_2_trans(j,i) = curprob_trans(2);
                %%% /TRANSIENt COPY
                probs(j, (2*i)-1) = p(1); % probability for non-speech stored to plot at end
                probs(j, 2*i) = p(2); % probability for speech stored to plot at end             
                %%%%%%%
                
            end %%% channel loop

            
            %%% UPDATE MODELS AS IN GMM ORG:
            mu = mu_frame;
            vari = vari_frame; %%% utilizes no constant from training
            wgt = wgt_frame; %%% utilizes no constant from training
            %%% /UPDATE MODELS AS IN GMM ORG
            
            mu_frame_veri(:, (2*i-1):(2*i)) = mu_frame(:,:); %%% store suggested updates for means (non-speech | speech) for verification purposes (only for testing)
            vari_frame_veri(:, (2*i-1):(2*i)) = vari_frame(:,:); %%% store suggested updates for means (non-speech | speech) for verification purposes (only for testing)
            wgt_frame_veri(:, (2*i-1):(2*i)) = wgt_frame(:,:); %%% store suggested updates for weights (non-speech | speech) for verification purposes (only for testing)
            
            mu_trans_veri(:, (2*i-1):(2*i)) = mu_trans(:,:); %%% store suggested updates for means (non-speech | speech) for verification purposes (only for testing)
            vari_trans_veri(:, (2*i-1):(2*i)) = vari_trans(:,:); %%% store suggested updates for means (non-speech | speech) for verification purposes (only for testing)
            wgt_trans_veri(:, (2*i-1):(2*i)) = wgt_trans(:,:); %%% store suggested updates for weights (non-speech | speech) for verification purposes (only for testing)
            
            for j = 1:num_chnl %%% classify all sub-bands individually
                y = z(j,i); %%% extract current frame's log-energy contour
                bdy(i, j) = deci_bdy(mu(j,:),vari(j,:),wgt(j,:), deciGamma);

                %%%%%%%%%%%%%%%%%% added  %%%%%%%%%%%%%%%%%%%%%%%%%%
                thres(i, j) = bdy(i, j);    %%% storage variables
                n_level(i, j) = min(mu(j,:)); % noise level
                s_level(i, j) = max(mu(j,:)); % speech level
                min_vari(i, j) = min(vari(j,:));
                max_vari(i, j) = max(vari(j,:));
                w_1(i, j) = wgt(1);
                w_2(i, j) = wgt(2);
                %%%%%%%%%%%%%%%%%% added  %%%%%%%%%%%%%%%%%%%%%%%%%%

%                 probs_1(j,i) = curprob(1);
%                 probs_2(j,i) = curprob(2);
%                 probs(j, (2*i)-1) = p(1); % probability for non-speech stored to plot at end
%                 probs(j, 2*i) = p(2); % probability for speech stored to plot at end

                if bdy(i, j) < y
                    idx(i,j)=1; %%% classification
                end
            end
            
            outid = sum(idx(i,:)*weights')/sum(weights); %%% sum over each channel, weighted.

            if outid>=thld
                idx_new(i) = 1; % idx originally
            end
            
            %%%hangover scheme from AMR
            if idx_new(i)==1
                outvad(i)=1;
                bc=bc+1;
                if bc>bth
                    hc=hth;
                end
            else
                bc=0;
                hc=hc-1;
                if hc<=0
                    outvad(i)=0;
                    hc=0;
                else
                    outvad(i)=1;
                end
            end
            
            %%% TRANSIENT COPY PARAMETER SET:
            ws(i) = sum(w.*wgt_frame(:,2)); %%% "probability" of speech
            
            if vadDecideUpdate
                if outvad(i) > 0
                    ws(i) = 1000; %%% speech "probability" is set "infinitely" high (guaranteed over SPF (max 1)).
                else
                    ws(i) = 0; %%% speech "probability" is set to zero.
                end
            end
            
            if ws(i) < speech_prob_floor %%%
                mu_trans(:,1) = mu_frame(1); %%% update only non-speech
            else
                mu_trans(:,:) = mu_frame; %%% update both components
            end
            if variFreeze == 1
                if ws(i) < speech_prob_floor %%%
                    vari_trans(:,1) = vari_frame(1); %%% update only non-speech
                else
                    vari_trans(:,:) = vari_frame; %%% update both components
                end
            else
                vari_trans = vari_frame; %%% utilizes no constant from training
            end
            if wgtFreeze == 1
                if ws(i) < speech_prob_floor %%%
                    wgt_trans(:,1) = wgt_frame(1); %%% update only non-speech
                else
                    wgt_trans(:,:) = wgt_frame; %%% update both components
                end
            else
                wgt_trans = wgt_frame; %%% utilizes no constant from training
            end
            %%% /TRANSIENT COPY PARAMETER SET
            
            %%% TRANSIENT DETECTION:
            %%% TRANSIENT COPY:
            silence_prob = sum(probs_1_trans(1:num_chnl,i),1); % Sum the probability from each subband
            speech_prob = sum(probs_2_trans(1:num_chnl,i),1); % Sum the probability from each subband
            %%% /TRANSIENT COPY
            ratio(i) = 1/(silence_prob + speech_prob);
            
            if transThresh >= 1 %%% use ecdf to find threshold if not directly given as input
                cdfThreshold(i) = transThresh;
            else
                [fcdf, xcdf] = ecdf(ratio(1:i)); %%% CDF estimate of ratios so far
                cdfThreshold(i) = xcdf(find(fcdf > transThresh,1)); %%% find the likelihood threshold for transient detection
            end
            
            if ratio(i) > cdfThreshold(i) %%% detected transient
                if transientRemainder > 0 %%% previously detected transient
                    if outvad(i) == speechFlag
                        outvad(i) = trsp; %%% marks frame as transient
                        transientRemainder = transientRemainder - 1;
                    else
                        outvad(i) = trnsp; %%% marks frame as transient
                        transientRemainder = transientRemainder - 1;
                    end
                else %%% transient initial attack detected
                    if outvad(i) == speechFlag
                        outvad(i) = trsp; %%% marks frame as transient
                        transientRemainder = transientLength; %%% set transient downward counter
                        transientRemainder = transientRemainder - 1;
                    else
                        outvad(i) = trsp; %%% marks frame as transient
                        transientRemainder = transientLength; %%% set transient downward counter
                        transientRemainder = transientRemainder - 1;
                    end
                end
            else
                if transientRemainder > 0 %%% no transient detected in current frame, but frame is in segment previously marked as transient 
                    if outvad(i) == speechFlag
                        outvad(i) = trsp; %%% marks frame as transient
                        transientRemainder = transientRemainder - 1;
                    else
                        outvad(i) = trnsp; %%% marks frame as transient
                        transientRemainder = transientRemainder - 1;
                    end
                end %%% else: do nothing to _outvad(i)_ 
            end
            %%% /TRANSIENT DETECTION
        end
    end %%% frame loop
    
    %%%
%     figure()
%     fr_sig = mean(z);
%     plot(fr_sig); hold on; plot(outvad, 'r');
    %%%x1 = 1:num; plotyy(x1,fr_sig, x1, outvad);
end