function [ out ] = infoTestFiles(testFiles)
%INFOTESTFILES:  Generates statistics from a cell array of test files generated by
%               _createTestFiles_.
%       Input: cell array as outputted by _createTestFiles_
%       Output: struct of statistics.
%
%       2014.05.19, Henrik Ldrup Parnemann

    noFiles = length(testFiles);
    fs = testFiles{1}.fs;
    lengthTot = 0; %%% total length
    noSegments = 0;
    sp = 0; %%% length of speech
    nsp = 0; %%% length of non-speech
    tr = 0; %%% length of transients
    ntr = 0; %%% length of non-transients
    spLength = 0; %%% length of speech segments (incl. potential silence)
    silLength = 0; %%% length of (absoulte) silence segments
    noSilSeg = 0; %%% number of silence segments
    noSpSeg = 0; %%% number of speech segments
    avgScale = 0;
    for i=1:noFiles
        lengthTot = lengthTot + length(testFiles{i}.concatenated);
        [~, ~, fixed, ~, ~] = createFrameAnnotation(testFiles{i}.starts, testFiles{i}.stops, length(testFiles{i}.concatenated), 160, 80, 2, fs, testFiles{i}.tranStart, testFiles{i}.tranStop); %%% see next line:
        %%% Not ideal to use _fixed_, as this is the annotation shifted
        %%% into frames. _org_ should have been used, had this also
        %%% included transient annotation. The error resulting from this
        %%% frame-shifting should not be of any significance, though, as
        %%% one may assume that the errors are equally distributed among
        %%% the non-speech and speech frames.
        
        %%% segments:
        for k = 1:length(testFiles{i}.segments)
            noSegments = noSegments + 1;
            if testFiles{i}.structure(k) %%% is pause segment
                noSilSeg = noSilSeg + 1;
                silLength = silLength + testFiles{i}.segments(k);
            else %%% is speech segment
                noSpSeg = noSpSeg + 1;
                spLength = spLength + testFiles{i}.segments(k);
            end
        end
        %%% speech / non-speech:
        sp = sp + length(find(fixed == 1)) + length(find(fixed == -1));
        nsp = nsp + length(find(fixed == 0)) + length(find(fixed == -2));
%         for l=1:length(testFiles{i}.starts)
%             nsp = nsp + testFiles{i}.stops(l) - testFiles{i}.starts(l);
%             if l == length(testFiles{i}.starts)
%                 sp = sp + length(testFiles{i}.concatenated) - testFiles{i}.starts(l);
%             else
%                 sp = sp + testFiles{i}.starts(l+1) - testFiles{i}.stops(l);
%             end
%         end
        %%% transients:
        tr = tr + length(find(fixed < 0));
        ntr = ntr + length(find(fixed >= 0));
%         for m=1:length(testFiles{i}.tranStart)
%             tr = tr + testFiles{i}.tranStop(m) - testFiles{i}.tranStart(m);
%             if m == length(testFiles{i}.tranStart)
%                 ntr = ntr + length(testFiles{i}.concatenated) - testFiles{i}.tranStop(m);
%             elseif m == 1
%                 ntr = ntr + testFiles{i}.tranStart(m) - 1;
%             else
%                 ntr = ntr + testFiles{i}.tranStart(m+1) - testFiles{i}.tranStop(m);
%             end
%         end
        avgScale = avgScale + mean(testFiles{i}.transientScaling)/noFiles;
    end
    
    %%% total:
    out.totalLength = lengthTot/fs; %%% length in seconds
    out.averageFileLength = lengthTot/(noFiles*fs); %%% length in seconds
    %%% speech / non-speech:
    out.speechRatio = sp/lengthTot;
    out.nonSpeechRatio = nsp/lengthTot;
%     out.nonSpeechRatio = 1 - sp/lengthTot;
    %%% segments:
    out.averageSegmentLength = lengthTot/(noSegments*fs); %%% length in seconds
    out.averageSilenceSegmentLength = silLength/(noSilSeg*fs); %%% length in seconds
    out.averageSpeechSegmentLength = spLength/(noSpSeg*fs); %%% length in seconds
    %%% transients:
    out.transientRatio = tr/lengthTot;
    out.nonTransientRatio = ntr/lengthTot;
    out.averageTransientScaling = avgScale;
     
end