 function  [mu,vari,wgt, idx]=iniGMM(x,delta)

%iniGMM - inilize the GMM with the constrained EM algorithm
% The parameters in this function is designed for signals with sampling
% rate 4k Hz.
% Inputs:
%   x       input samples
%   delta   constraints to the means
%
% Outputs:
%   mu      mean
%   vari    variance
%   wgt     weight coefficient
% reference paper: D. Ying, Y. Yan, J. Dang, and F. Soong, "Voice activity detection based on unsupervised learning
% framework," IEEE Trans. Audio, Speech and Language Processing, 19(8), Nov,
% 2011
%June-15
%Dongwen Ying

num = length(x); %%% length M from gmmVad()

% use k-means to initialize the EM-algorithm
[idx,mu] = kmeans_new(x,2);   % cluster the 60 first frames into 2 clusters using k-means clustering %%% speech / non-speech

wei(1:2) = 0.5;

% -------------------- original code -------------------------------------
% pos = find(idx==1);
% vari(1)= var(x(pos)); % initialization before the EM-algorithm
% pos = find(idx==2);
% vari(2)= var(x(pos)); % initialization before the EM-algorithm
% ------------------------------------------------------------------------

% -------------------- corrected code ------------------------------------
x_sort = sort(x);
pos = find(idx==1);
vari(1)= var(x_sort(pos)); % initialization before the EM-algorithm
pos = find(idx==2);
vari(2)= var(x_sort(pos)); % initialization before the EM-algorithm
% ------------------------------------------------------------------------

mu=mu(:);
if mu(1)>mu(2)
    mu = flipud(mu);
    vari = fliplr(vari);
    wei = fliplr(wei);
end

thisprob = -1.0e+30/2; lastprob = -1.0e+30;
iter = 0;ThresNum=0;
ii=0;

% EM-algorithm that ends when the algorithm starts to converge.
while iter<20 && ThresNum<3 % max 20 iteration or convergence
    if ((thisprob - lastprob) / abs(thisprob) < 0.001)
        ThresNum=ThresNum+1;
    else
        ThresNum = 0;
    end
    
    newbook(1:2) = 0;
    newvari(1:2) = 0;
    newwgt(1:2) = 0;
    
    lastprob = thisprob;
    thisprob = 0;
    for cnt=1:num % 1 : 60
        curprob(1,cnt) = wei(1)*0.3989/sqrt(vari(1))*exp(-0.5*(x(cnt)-mu(1))^2/vari(1));
        curprob(2,cnt) = wei(2)*0.3989/sqrt(vari(2))*exp(-0.5*(x(cnt)-mu(2))^2/vari(2));
        tmp = curprob(1,cnt)+curprob(2,cnt);
        curprob(1,cnt) = curprob(1,cnt)/tmp; % eq. 8
        curprob(2,cnt) = curprob(2,cnt)/tmp;
        
        thisprob = thisprob + log(tmp);
        
        TmpFloat = curprob(1,cnt) * x(cnt); 
        newbook(1) = newbook(1)+TmpFloat; % numerator eq. 6
        newvari(1) = newvari(1)+TmpFloat * x(cnt); % part of numerator eq. 7
        newwgt(1) = newwgt(1)+curprob(1,cnt); % numerator eq. 5
        
        TmpFloat = curprob(2,cnt) * x(cnt);
        newbook(2) = newbook(2)+TmpFloat;
        newvari(2) = newvari(2)+TmpFloat * x(cnt);
        newwgt(2) = newwgt(2)+curprob(2,cnt);
    end
    ii = ii + 1;
    prob(ii) = thisprob;
    
    TmpFloat = newbook(1) / newwgt(1);
    mu(1) = TmpFloat; % ok
    vari(1) = newvari(1) / newwgt(1) - TmpFloat * TmpFloat;
    wei(1) = newwgt(1) / num;
    
    TmpFloat = newbook(2) / newwgt(2);
    mu(2) = max(TmpFloat, mu(1) + delta); % constraint from eq. 20
    vari(2) = newvari(2) / newwgt(2) - TmpFloat * TmpFloat;
    wei(2) = newwgt(2) / num;
    iter = iter + 1;
    
    if wei(1) / wei(2) > 8 % not exactly the same as the constraint described in the paper
        break;
    end
end


wgt = wei;
mu = mu';