function out = makeROC(anno, gmm, hlp, itu, varargin)
%%% Heavuly modified version of _roc_plot.m_
% This function calculates the ROCs for each method.
%%% UPDATED 2014.05.12 %%%
%%% UPDATE 2014.05.28: TRANSIENTS SHOULD BE CLASSIFIED AS SPEECH, REGARDLESS OF PREVIOUS CONTENT! CHANGES ARE MARKED WITH "%%% 28.05: ".

    %%% FLAGS:
    if isempty(varargin)
        sp = 1; %%% speech
        nsp = 0; %%% non-speech
        trsp = -1; %%% transient in speech
        trnsp = -2; %%% transient in non-speech
    else
        sp = varargin{1}{1,2};
        nsp = varargin{1}{2,2};
        trsp = varargin{1}{3,2};
        trnsp = varargin{1}{4,2};
    end
    
    if length(varargin) == 2
        so = 1; %%% sohn vad is used
        sohn = varargin{2};
    else
        so = 0;
        sohn = [];
    end

    x_len = min([length(anno) length(gmm) length(hlp) length(itu) length(sohn)]);

    tpr1 = 0; fpr1 = 0; tnr1 = 0; fnr1 = 0; %%% GMM VAD: true/false positive/negative rate
    tpr2 = 0; fpr2 = 0; tnr2 = 0; fnr2 = 0; %%% HLP VAD: true/false positive/negative rate
    tpr3 = 0; fpr3 = 0; tnr3 = 0; fnr3 = 0; %%% ITU VAD: true/false positive/negative rate
    tpr4 = 0; fpr4 = 0; tnr4 = 0; fnr4 = 0; %%% GMM transient detection: true/false positive/negative rate
    tpr5 = 0; fpr5 = 0; tnr5 = 0; fnr5 = 0; %%% HLP transient detection: true/false positive/negative rate
    tpr6 = 0; fpr6 = 0; tnr6 = 0; fnr6 = 0; %%% ITU VAD: true/false positive/negative rate
    
    speechGMM = 0; %%% no. of (true) speech frames
    non_speechGMM = 0; %%% no. of (true) non-speech frames
    transientGMM = 0; %%% no. of (true) transient frames
    non_transientGMM = 0; %%% no. of (true) non-transient frames
    
    speechHLP = 0; %%% no. of (true) speech frames
    non_speechHLP = 0; %%% no. of (true) non-speech frames 
    transientHLP = 0; %%% no. of (true) transient frames
    non_transientHLP = 0; %%% no. of (true) non-transient frames
    
    speechITU = 0; %%% no. of (true) speech frames
    non_speechITU = 0; %%% no. of (true) non-speech frames
    
    speechSohn = 0; %%% no. of (true) speech frames
    non_speechSohn = 0; %%% no. of (true) non-speech frames 

    %figure, plot(anno_VAD), hold on , plot(org_VAD*0.3,'r')

    for i = 1 : x_len
        %%% GMM:
        %%% 28.05: if (anno(i) == sp || anno(i) == trsp) && gmm(i) == sp %%% true positive detected
        if (anno(i) == sp || anno(i) == trsp || anno(i) == trnsp) && gmm(i) == sp %%% true positive detected
            tpr1 = tpr1 + 1;
            speechGMM = speechGMM + 1;
        %%% 28.05: elseif (anno(i) == nsp || anno(i) == trnsp) && gmm(i) == sp %%% false positive detected
        elseif (anno(i) == nsp) && gmm(i) == sp %%% false positive detected
            fpr1 = fpr1 + 1;
            non_speechGMM = non_speechGMM + 1;
        %%% 28.05: elseif (anno(i) == nsp || anno(i) == trnsp) && gmm(i) == nsp %%% true negative detected
        elseif (anno(i) == nsp) && gmm(i) == nsp %%% true negative detected
            tnr1 = tnr1 + 1;
            non_speechGMM = non_speechGMM + 1;
        %%% 28.05: elseif (anno(i) == sp || anno(i) == trsp) && gmm(i) == nsp %%% false negative detected
        elseif (anno(i) == sp || anno(i) == trsp || anno(i) == trnsp) && gmm(i) == nsp %%% false negative detected
            fnr1 = fnr1 + 1;
            speechGMM = speechGMM + 1;
        end

        %%% GMM transient:
        if (anno(i) == trsp || anno(i) == trnsp) && (gmm(i) == trsp || gmm(i) == trnsp) %%% true positive detected
            tpr4 = tpr4 + 1;
            transientGMM = transientGMM + 1;
        elseif (anno(i) == sp || anno(i) == nsp) && (gmm(i) == trsp || gmm(i) == trnsp) %%% false positive detected
            fpr4 = fpr4 + 1;
            non_transientGMM = non_transientGMM + 1;
        elseif (anno(i) == trsp || anno(i) == trnsp) && (gmm(i) == sp || gmm(i) == nsp) %%% true negative detected
            fnr4 = fnr4 + 1;
            transientGMM = transientGMM + 1;
        elseif (anno(i) == sp || anno(i) == nsp) && (gmm(i) == sp || gmm(i) == nsp) %%% false negative detected
            tnr4 = tnr4 + 1;
            non_transientGMM = non_transientGMM + 1;
        end        
        
        %%% GMM HLP:
        %%% 28.05: if (anno(i) == sp || anno(i) == trsp) && hlp(i) == sp %%% true positive detected
        if (anno(i) == sp || anno(i) == trsp || anno(i) == trnsp) && hlp(i) == sp %%% true positive detected
            tpr2 = tpr2 + 1;
            speechHLP = speechHLP + 1;
        %%% 28.05: elseif (anno(i) == nsp || anno(i) == trnsp) && hlp(i) == sp %%% false positive detected
        elseif (anno(i) == nsp) && hlp(i) == sp %%% false positive detected
            fpr2 = fpr2 + 1;
            non_speechHLP = non_speechHLP + 1;
        %%% 28.05: elseif (anno(i) == nsp || anno(i) == trnsp) && hlp(i) == nsp %%% true negative detected
        elseif (anno(i) == nsp) && hlp(i) == nsp %%% true negative detected
            tnr2 = tnr2 + 1;
            non_speechHLP = non_speechHLP + 1;
        %%% 28.05: elseif (anno(i) == sp || anno(i) == trsp) && hlp(i) == nsp %%% false negative detected
        elseif (anno(i) == sp || anno(i) == trsp || anno(i) == trnsp) && hlp(i) == nsp %%% false negative detected
            fnr2 = fnr2 + 1;
            speechHLP = speechHLP + 1;
        end
        
        %%% GMM HLP transient:
        if (anno(i) == trsp || anno(i) == trnsp) && (hlp(i) == trsp || hlp(i) == trnsp) %%% true positive detected
            tpr5 = tpr5 + 1;
            transientHLP = transientHLP + 1;
        elseif (anno(i) == sp || anno(i) == nsp) && (hlp(i) == trsp || hlp(i) == trnsp) %%% false positive detected
            fpr5 = fpr5 + 1;
            non_transientHLP = non_transientHLP + 1;
        elseif (anno(i) == trsp || anno(i) == trnsp) && (hlp(i) == sp || hlp(i) == nsp) %%% true negative detected
            fnr5 = fnr5 + 1;
            transientHLP = transientHLP + 1;
        elseif (anno(i) == sp || anno(i) == nsp) && (hlp(i) == sp || hlp(i) == nsp) %%% false negative detected
            tnr5 = tnr5 + 1;
            non_transientHLP = non_transientHLP + 1;
        end   
        
        %%% ITU:
        %%% 28.05: if (anno(i) == sp || anno(i) == trsp) && itu(i) == sp %%% true positive detected
        if (anno(i) == sp || anno(i) == trsp || anno(i) == trnsp) && itu(i) == sp %%% true positive detected
            tpr3 = tpr3 + 1;
            speechITU = speechITU + 1;
        %%% 28.05: elseif (anno(i) == nsp || anno(i) == trnsp) && itu(i) == sp %%% false positive detected
        elseif (anno(i) == nsp) && itu(i) == sp %%% false positive detected
            fpr3 = fpr3 + 1;
            non_speechITU = non_speechITU + 1;
        %%% 28.05: elseif (anno(i) == nsp || anno(i) == trnsp) && itu(i) == nsp %%% true negative detected
        elseif (anno(i) == nsp) && itu(i) == nsp %%% true negative detected
            tnr3 = tnr3 + 1;
            non_speechITU = non_speechITU + 1;
        %%% 28.05: elseif (anno(i) == sp || anno(i) == trsp) && itu(i) == nsp %%% false negative detected
        elseif (anno(i) == sp || anno(i) == trsp || anno(i) == trnsp) && itu(i) == nsp %%% false negative detected
            fnr3 = fnr3 + 1;
            speechITU = speechITU + 1;
        end
        
        if so
            %%% Sohn:
            %%% 28.05: if (anno(i) == sp || anno(i) == trsp) && itu(i) == sp %%% true positive detected
            if (anno(i) == sp || anno(i) == trsp || anno(i) == trnsp) && sohn(i) == sp %%% true positive detected
                tpr6 = tpr6 + 1;
                speechSohn = speechSohn + 1;
            %%% 28.05: elseif (anno(i) == nsp || anno(i) == trnsp) && itu(i) == sp %%% false positive detected
            elseif (anno(i) == nsp) && sohn(i) == sp %%% false positive detected
                fpr6 = fpr6 + 1;
                non_speechSohn = non_speechSohn + 1;
            %%% 28.05: elseif (anno(i) == nsp || anno(i) == trnsp) && itu(i) == nsp %%% true negative detected
            elseif (anno(i) == nsp) && sohn(i) == nsp %%% true negative detected
                tnr6 = tnr6 + 1;
                non_speechSohn = non_speechSohn + 1;
            %%% 28.05: elseif (anno(i) == sp || anno(i) == trsp) && itu(i) == nsp %%% false negative detected
            elseif (anno(i) == sp || anno(i) == trsp || anno(i) == trnsp) && sohn(i) == nsp %%% false negative detected
                fnr6 = fnr6 + 1;
                speechSohn = speechSohn + 1;
            end
        end
    end

    tpr1 = tpr1 / speechGMM; tnr1 = tnr1 / non_speechGMM;
    tpr2 = tpr2 / speechHLP; tnr2 = tnr2 / non_speechHLP;
    tpr3 = tpr3 / speechITU; tnr3 = tnr3 / non_speechITU;
    tpr4 = tpr4 / transientGMM; tnr4 = tnr4 / non_transientGMM;
    tpr5 = tpr5 / transientHLP; tnr5 = tnr5 / non_transientHLP;
    tpr6 = tpr6 / speechSohn; tnr6 = tnr6 / non_speechSohn;

    %disp(tis)
    %disp(1-nt)
    %disp('')
    
    if so
        out = [tnr1, tpr1;
               tnr2, tpr2; 
               tnr3, tpr3; 
               tnr4, tpr4;
               tnr5, tpr5;
               tnr6, tpr6];
    else
        out = [tnr1, tpr1;
               tnr2, tpr2; 
               tnr3, tpr3; 
               tnr4, tpr4;
               tnr5, tpr5];
    end
    
%%% WAS (2014.05.12):
%         if anno(i) > 0 && gmm(i) > 0
%             tpr1 = tpr1 + 1;
%             speechGMM = speechGMM + 1;
%         elseif anno(i) == 0 && gmm(i) > 0
%             fpr1 = fpr1 + 1;
%             non_speechGMM = non_speechGMM + 1;
%         elseif anno(i) == 0 && gmm(i) == 0
%             tnr1 = tnr1 + 1;
%             non_speechGMM = non_speechGMM + 1;
%         elseif anno(i) > 0 && gmm(i) == 0
%             fnr1 = fnr1 + 1;
%             speechGMM = speechGMM + 1;
%         end
%%% /WAS (2014.05.12)
end