function [ ] = plotTestFiles(testFiles)
%PLOTTESTFILES:  Plots a cell array of test files generated by
%               _createTestFiles_.
%       Input: cell array as outputted by _createTestFiles_
%       Output: none
%
%       2014.05.13, Henrik Ldrup Parnemann


    noFilesTot = length(testFiles);
    for ii = 1:noFilesTot
        if ((mod((ii-1), 2)+1) == 1) %%% plot two files in each figure window
            figure();
        end
        fs = testFiles{ii}.fs;
        subplot(2,1,(mod((ii-1), 2)+1))
        xax = (1:length(testFiles{ii}.concatenated))./(fs);
        con = plot(xax, testFiles{ii}.concatenated, 'Color', [0.3 0.3 0.3]);
        hold on;
        %%% plot segment starts
        for i = 1:length(testFiles{ii}.shifts) %%% same length for all files
            seg(i) = plot([(testFiles{ii}.shifts(i)+1)/fs, (testFiles{ii}.shifts(i)+1)/fs],[-0.8 0.8], 'r', 'LineWidth', 2);
%             ylim([-0.8 0.8]);
        end
        for i = 1:length(testFiles{ii}.starts)
            str(i) = plot([(testFiles{ii}.starts(i))/fs, (testFiles{ii}.starts(i))/fs],[-0.7 0.7], 'b--', 'LineWidth', 2);
%             ylim([-0.6 0.6]);
        end
        for i = 1:length(testFiles{ii}.stops)
            stp(i) = plot([(testFiles{ii}.stops(i))/fs, (testFiles{ii}.stops(i))/fs],[-0.6 0.6], 'g--', 'LineWidth', 2);
%             ylim([-0.5 0.5]);
        end
        for i = 1:length(testFiles{ii}.tranStart)
            trstr(i) = plot([(testFiles{ii}.tranStart(i))/fs, (testFiles{ii}.tranStart(i))/fs],[-0.5 0.5], 'Color', [0.7 0.8 0.5], 'LineWidth', 1.5);
%             ylim([-0.8 0.8]);
        end
        for i = 1:length(testFiles{ii}.tranStop)
            trstp(i) = plot([(testFiles{ii}.tranStop(i))/fs, (testFiles{ii}.tranStop(i))/fs],[-0.4 0.4], '-.', 'Color', [0.5 0.4 0.5], 'LineWidth', 1.5);
%             ylim([-0.8 0.8]);
        end
        if ((mod((ii-1), 2)+1) == 1)
            title(['File no. ', num2str(ii), ' & ', num2str(ii+1)]);
        end
        if ((mod((ii-1), 2)+1) == 2)
            xlabel('Seconds');
        end
        ylabel('y(t)');
        xlim([0 1.01*length(testFiles{ii}.concatenated)/fs]);
        ylim([-0.8 0.8]);
        legend([seg(1) con str(1) stp(1) trstr(1) trstp(1)], 'Segment starts', 'Concatenated files', 'Silence start', 'Silence stop', 'Transient start','Transient stop')
    end
end

